@@ -1,4 +1,4 @@
-__version__ = "3.8.0"
+__version__ = "3.11.2"
 
 from .advertisement_tracker import (
     TRACKER_BUFFERING_WOBBLE_SECONDS,
@@ -18,6 +18,7 @@ from .models import (
     BluetoothServiceInfo,
     BluetoothServiceInfoBleak,
     HaBluetoothConnector,
+    HaBluetoothSlotAllocations,
 )
 from .scanner import BluetoothScanningMode, HaScanner, ScannerStartError
 from .scanner_device import BluetoothScannerDevice
@@ -41,6 +42,7 @@ __all__ = [
     "HaBleakClientWrapper",
     "HaBleakScannerWrapper",
     "HaBluetoothConnector",
+    "HaBluetoothSlotAllocations",
     "HaScanner",
     "ScannerStartError",
     "get_manager",
