@@ -10,7 +10,12 @@ from functools import partial
 from typing import TYPE_CHECKING, Any, Final
 
 from bleak.backends.scanner import AdvertisementDataCallback
-from bleak_retry_connector import NO_RSSI_VALUE, RSSI_SWITCH_THRESHOLD, BleakSlotManager
+from bleak_retry_connector import (
+    NO_RSSI_VALUE,
+    AllocationChangeEvent,
+    Allocations,
+    BleakSlotManager,
+)
 from bluetooth_adapters import (
     ADAPTER_ADDRESS,
     ADAPTER_PASSIVE_SCAN,
@@ -24,12 +29,13 @@ from .advertisement_tracker import (
     AdvertisementTracker,
 )
 from .const import (
+    ADV_RSSI_SWITCH_THRESHOLD,
     CALLBACK_TYPE,
     FAILED_ADAPTER_MAC,
     FALLBACK_MAXIMUM_STALE_ADVERTISEMENT_SECONDS,
     UNAVAILABLE_TRACK_SECONDS,
 )
-from .models import BluetoothServiceInfoBleak
+from .models import BluetoothServiceInfoBleak, HaBluetoothSlotAllocations
 from .scanner_device import BluetoothScannerDevice
 from .usage import install_multiple_bleak_catcher, uninstall_multiple_bleak_catcher
 from .util import async_reset_adapter
@@ -95,11 +101,15 @@ class BluetoothManager:
 
     __slots__ = (
         "_adapter_refresh_future",
+        "_adapter_sources",
         "_adapters",
         "_advertisement_tracker",
         "_all_history",
+        "_allocations",
+        "_allocations_callbacks",
         "_bleak_callbacks",
         "_bluetooth_adapters",
+        "_cancel_allocation_callbacks",
         "_cancel_unavailable_tracking",
         "_connectable_history",
         "_connectable_scanners",
@@ -142,15 +152,25 @@ class BluetoothManager:
         self._non_connectable_scanners: set[BaseHaScanner] = set()
         self._connectable_scanners: set[BaseHaScanner] = set()
         self._adapters: dict[str, AdapterDetails] = {}
+        self._adapter_sources: dict[str, str] = {}
+        self._allocations: dict[str, HaBluetoothSlotAllocations] = {}
         self._sources: dict[str, BaseHaScanner] = {}
         self._bluetooth_adapters = bluetooth_adapters
         self.slot_manager = slot_manager
+        self._cancel_allocation_callbacks = (
+            self.slot_manager.register_allocation_callback(
+                self._async_slot_manager_changed
+            )
+        )
         self._debug = _LOGGER.isEnabledFor(logging.DEBUG)
         self.shutdown = False
         self._loop: asyncio.AbstractEventLoop | None = None
         self._adapter_refresh_future: asyncio.Future[None] | None = None
         self._recovery_lock: asyncio.Lock = asyncio.Lock()
         self._disappeared_callbacks: set[Callable[[str], None]] = set()
+        self._allocations_callbacks: dict[
+            str | None, set[Callable[[HaBluetoothSlotAllocations], None]]
+        ] = {}
 
     @property
     def supports_passive_scan(self) -> bool:
@@ -202,13 +222,7 @@ class BluetoothManager:
     ) -> CALLBACK_TYPE:
         """Register a callback to be called when an address disappears."""
         self._disappeared_callbacks.add(callback)
-        return partial(self._async_remove_disappeared_callback, callback)
-
-    def _async_remove_disappeared_callback(
-        self, callback: Callable[[str], None]
-    ) -> None:
-        """Remove a disappeared callback."""
-        self._disappeared_callbacks.discard(callback)
+        return partial(self._disappeared_callbacks.discard, callback)
 
     async def _async_refresh_adapters(self) -> None:
         """Refresh the adapters."""
@@ -282,6 +296,7 @@ class BluetoothManager:
             self._cancel_unavailable_tracking.cancel()
             self._cancel_unavailable_tracking = None
         uninstall_multiple_bleak_catcher()
+        self._cancel_allocation_callbacks()
 
     def async_scanner_devices_by_address(
         self, address: str, connectable: bool
@@ -430,10 +445,10 @@ class BluetoothManager:
                     stale_seconds,
                 )
             return False
-        if (new.rssi or NO_RSSI_VALUE) - RSSI_SWITCH_THRESHOLD > (
+        if (new.rssi or NO_RSSI_VALUE) - ADV_RSSI_SWITCH_THRESHOLD > (
             old.rssi or NO_RSSI_VALUE
         ):
-            # If new advertisement is RSSI_SWITCH_THRESHOLD more,
+            # If new advertisement is ADV_RSSI_SWITCH_THRESHOLD more,
             # the new one is preferred.
             if self._debug:
                 _LOGGER.debug(
@@ -446,7 +461,7 @@ class BluetoothManager:
                     self._async_describe_source(old),
                     self._async_describe_source(new),
                     new.rssi,
-                    RSSI_SWITCH_THRESHOLD,
+                    ADV_RSSI_SWITCH_THRESHOLD,
                     old.rssi,
                 )
             return False
@@ -689,6 +704,8 @@ class BluetoothManager:
         self._advertisement_tracker.async_remove_source(scanner.source)
         scanners.remove(scanner)
         del self._sources[scanner.source]
+        del self._adapter_sources[scanner.adapter]
+        self._allocations.pop(scanner.source, None)
         if connection_slots:
             self.slot_manager.remove_adapter(scanner.adapter)
 
@@ -705,6 +722,7 @@ class BluetoothManager:
             scanners = self._non_connectable_scanners
         scanners.add(scanner)
         self._sources[scanner.source] = scanner
+        self._adapter_sources[scanner.adapter] = scanner.source
         if connection_slots:
             self.slot_manager.register_adapter(scanner.adapter, connection_slots)
         return partial(
@@ -748,3 +766,57 @@ class BluetoothManager:
     ) -> None:
         """Override the fallback availability timeout for a MAC address."""
         self._fallback_intervals[address] = interval
+
+    def _async_slot_manager_changed(self, event: AllocationChangeEvent) -> None:
+        """Handle slot manager changes."""
+        self.async_on_allocation_changed(
+            self.slot_manager.get_allocations(event.adapter)
+        )
+
+    def async_on_allocation_changed(self, allocations: Allocations) -> None:
+        """Call allocation callbacks."""
+        source = self._adapter_sources.get(allocations.adapter, allocations.adapter)
+        ha_slot_allocations = HaBluetoothSlotAllocations(
+            source=source,
+            slots=allocations.slots,
+            free=allocations.free,
+            allocated=allocations.allocated,
+        )
+        self._allocations[source] = ha_slot_allocations
+        for source_key in (source, None):
+            if not (
+                allocation_callbacks := self._allocations_callbacks.get(source_key)
+            ):
+                continue
+            for callback_ in allocation_callbacks:
+                try:
+                    callback_(ha_slot_allocations)
+                except Exception:
+                    _LOGGER.exception("Error in allocation callback")
+
+    def async_current_allocations(
+        self, source: str | None = None
+    ) -> list[HaBluetoothSlotAllocations] | None:
+        """Return the current allocations."""
+        if source:
+            if allocations := self._allocations.get(source):
+                return [allocations]
+            return []
+        return list(self._allocations.values())
+
+    def async_register_allocation_callback(
+        self,
+        callback: Callable[[HaBluetoothSlotAllocations], None],
+        source: str | None = None,
+    ) -> CALLBACK_TYPE:
+        """Register a callback to be called when an allocations change."""
+        self._allocations_callbacks.setdefault(source, set()).add(callback)
+        return partial(self._async_unregister_allocation_callback, callback, source)
+
+    def _async_unregister_allocation_callback(
+        self, callback: Callable[[HaBluetoothSlotAllocations], None], source: str | None
+    ) -> None:
+        if (callbacks := self._allocations_callbacks.get(source)) is not None:
+            callbacks.discard(callback)
+            if not callbacks:
+                del self._allocations_callbacks[source]
